@@ -1,29 +1,71 @@
 """
 Created by Mike Kittridge on 2020-11-02.
 
-For hashing the geometry of stations, use blake2b with a digest_size of 12.
-Similar for the dataset_id, except that the first 8 fields (starting with feature) should be used for the hashing.
+For hashing the geometry of stations, use blake2b with a digest_size of 12 of the hex encoded WKB.
+Similar for the dataset_id, except that the first 8 fields (starting with feature) of the dataset dict should be used for the hashing.
 """
 from datetime import datetime, date
 from typing import List, Optional, Dict, Union
 from pydantic import BaseModel, Field, HttpUrl
 import orjson
-from .utils import orjson_dumps
-from . import base
-from .geometry import geometry
+from enum import Enum
+from tethys_data_models.utils import orjson_dumps
+from tethys_data_models import base
+from tethys_data_models.geometry import geometry
 
 #########################################
 ### Models
 
 
-class ResultsDimensions(BaseModel):
-    lat: int = None
-    lon: int = None
-    geometry: int = None
+class ResultType(str, Enum):
+    time_series = 'time_series'
+    grid = 'grid'
+    trajectory = 'trajectory'
+    time_series_bands = 'time_series_bands'
+    grid_bands = 'grid_bands'
+    trajectory_bands = 'trajectory_bands'
+
+
+class TimeSeriesDims(BaseModel):
+    geometry: int
     height: int
     time: int
 
 
+class GridDims(BaseModel):
+    lat: int
+    lon: int
+    height: int
+    time: int
+
+
+class TrajectoryDims(BaseModel):
+    geometry: int
+
+
+class TimeSeriesBandsDims(BaseModel):
+    geometry: int
+    height: int
+    time: int
+    band: int
+
+
+class GridBandsDims(BaseModel):
+    lat: int
+    lon: int
+    height: int
+    time: int
+    band: int
+
+
+class TrajectoryBandsDims(BaseModel):
+    geometry: int
+    band: int
+
+
+ResultDims = Union[TimeSeriesDims, GridDims, TrajectoryDims, TimeSeriesBandsDims, GridBandsDims, TrajectoryBandsDims]
+
+
 class TimeRange(BaseModel):
     """
 
@@ -43,49 +85,74 @@ class Stats(BaseModel):
     count: int
 
 
-class S3ObjectKey(BaseModel):
+class ResultChunk(BaseModel):
     """
-    S3 object key and associated metadata.
+    Contain the data about a chunk object of a specific result.
     """
     key: str
     bucket: str
     content_length: int
-    etag: str = None
+    etag: str
     run_date: datetime
+    heights: Union[List[float], List[int]]
+    time_range: TimeRange = Field(..., description='The maximum time range of the result chunk.')
+
+
+class ResultVersion(BaseModel):
+    """
+    Groups the result chunks into an object. Contains information about a specific result version.
+    """
+    name: str = None
+    run_date: datetime
+    content_length: int
+    doi: str = None
+    description: str = Field(None, description='Description of the results version.')
+    result_chunks: List[ResultChunk]
+
+
+class ResultVersionGroup(BaseModel):
+    """
+    Groups many result versions together with the dataset_id and station_id. This is to be saved to the *.results_versions.json.zst object.
+    """
+    dataset_id: str = Field(..., description='The dataset uuid.')
+    station_id: str = Field(..., description='station id based on the geometry')
+    result_versions: List[ResultVersion]
+
+    class Config:
+        json_loads = orjson.loads
+        json_dumps = orjson_dumps
 
 
 class StationBase(base.Station):
     """
-    Contains the base station data.
+    Contains the base station data. This is used during a processing step before other data is available (i.e. result_versions and modified_date).
     """
-    dataset_id: str = Field(..., description='The unique dataset uuid.')
-    dimensions: ResultsDimensions
+    dataset_id: str = Field(..., description='The dataset uuid.')
+    dimensions: ResultDims
     heights: Union[List[float], List[int]]
-    # heights: List[int]
     time_range: TimeRange = Field(..., description='The maximum time range of the result.')
     stats: Stats
-    properties: Dict = Field(None, description='Any additional station specific properties.')
 
 
 class Station(StationBase):
     """
     Contains the complete station data.
     """
-    results_object_key: List[S3ObjectKey]
+    result_versions: List[ResultVersion]
     modified_date: datetime = Field(..., description='The modification date of the last edit.')
 
 
 class StationAgg(StationBase):
     """
-    Contains the complete station data for the combined stations in the entire dataset.
+    Contains the complete station data, but only includes the most recent results version. This object is meant to be combined with all of the other stations in a dataset (i.e. the *.stations.json.zst file).
     """
-    results_object_key: S3ObjectKey
+    result_versions: ResultVersion
     modified_date: datetime = Field(..., description='The modification date of the last edit.')
 
 
 class DatasetBase(BaseModel):
     """
-    Core fields in the dataset to create the dataset id.
+    Core fields in the dataset metadata to create the dataset_id.
     """
     feature: str = Field(..., description='The hydrologic feature associated with the dataset.')
     parameter: str = Field(..., description='The recorded observation parameter.')
@@ -99,13 +166,14 @@ class DatasetBase(BaseModel):
 
 class Dataset(DatasetBase):
     """
-    Dataset scheama.
+    Full dataset metadata schema.
     """
     dataset_id: str = Field(..., description='The unique dataset id based on the felds in the DatasetBase model.')
     units: str = Field(..., description='The units of the result.')
     license: str = Field(..., description='The legal data license associated with the dataset defined by the owner.')
     attribution: str = Field(..., description='The legally required attribution text to be distributed with the data defined by the owner.')
-    spatial_distribution: str = Field(..., description='This describes how the spatial data are distributed. Either sparse or grid.')
+    result_type: str = Field(..., description='This describes how the results are structurally stored.')
+    # spatial_distribution: str = Field(..., description='This describes how the spatial data are distributed. Either sparse or grid.')
     geometry_type: str = Field(..., description='This describes how the spatial dimensions are stored. Point, Line, Polygon, or Collection. Follows the OGC spatial data types.')
     grouping: str = Field(..., description='This describes how the staions and the associated data are grouped. Either none or blocks.')
     extent: geometry = Field(None, description='The geographical extent of the datset as a simple rectangular polygon.')
@@ -128,6 +196,31 @@ class Dataset(DatasetBase):
         json_dumps = orjson_dumps
 
 
+class ResultAttrs(BaseModel):
+    """
+    The result attributes that should be in the results netcdf file.
+    """
+    result_type: ResultType
+    title: str
+    insttution: str
+    license: str
+    source: str
+    history: str
+    version: int
+
+
+# class DataVars(BaseModel):
+#     """
+
+#     """
+
+
+
+# class ResultBase(BaseModel):
+#     """
+#     Core fields of the result.
+#     """
+#     # attrs: ResultAttrs
 
 
 
